import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Hotel, Booking, Room, UserState } from '../../types';
import { User } from '../../types';

@Injectable({
  providedIn: 'root',
})
export class ClienteApiRestService {
  private static readonly BASE_URI = 'http://localhost:8080';
  private static readonly HOTEL_URI = `${ClienteApiRestService.BASE_URI}/hotels`;
  private static readonly USER_URI = `${ClienteApiRestService.BASE_URI}/users`;
  constructor(private http: HttpClient) {}

  getHotel(id: number) {
    const url = `${ClienteApiRestService.HOTEL_URI}/${id}`;
    return this.http.get<Hotel>(url);
  }

  getAllHotels() {
    const url = `${ClienteApiRestService.HOTEL_URI}`;
    return this.http.get<Hotel[]>(url);
  }

  deleteHotel(id: number) {
    const url = `${ClienteApiRestService.HOTEL_URI}/${id}`;
    return this.http.delete(url, { observe: 'response', responseType: 'text' });
  }

  addHotel(hotel: Hotel) {
    const url = `${ClienteApiRestService.HOTEL_URI}`;
    return this.http.post(url, hotel, {
      observe: 'response',
      responseType: 'text',
    });
  }

  alterRoomAvailability(
    hotelId: number,
    roomId: number,
    availability: boolean
  ) {
    const url = `${ClienteApiRestService.HOTEL_URI}/${hotelId}/rooms/${roomId}`;
    return this.http.patch(
      url,
      { available: availability },
      {
        observe: 'response',
        responseType: 'text',
      }
    );
  }

  createBooking(bookingRequest: Booking) {
    return this.http.post('http://localhost:8080/bookings', bookingRequest);
  }

  getRoomsAvailableInDateRange(hotelId: number, start: Date, end: Date) {
    const startStr = start.toISOString().split('T')[0];
    const endStr = end.toISOString().split('T')[0];
    const url = `${ClienteApiRestService.HOTEL_URI}/${hotelId}/rooms?start=${startStr}&end=${endStr}`;
    return this.http.get<Room[]>(url);
  }

  getUser(userId: number) {
    return this.http.get<User>(`http://localhost:8080/users/${userId}`);
  }

  getAllUsers() {
    return this.http.get<User[]>('http://localhost:8080/users', {
      observe: 'body',
    });
  }

  getUserBookings(userId: number) {
    return this.http.get<Booking[]>(
      `${ClienteApiRestService.BASE_URI}/users/${userId}/bookings`
    );
  }

  alterUserStatus(userId: number, status: UserState) {
    return this.http.patch(
      `${ClienteApiRestService.BASE_URI}/users/${userId}`,
      {
        status,
      },
      {
        observe: 'response',
        responseType: 'text',
      }
    );
  }
}
